#!/bin/bash

LOG_FILE="/var/log/pptp-vpn-setup.log"
exec > >(tee -a "$LOG_FILE") 2>&1

echo "📦 Установка PPTP VPN сервера"
echo "📅 $(date)"

# Проверка прав
if [ "$(id -u)" -ne 0 ]; then
  echo "❌ Запусти этот скрипт от root!"
  exit 1
fi

# Выбор DNS
echo "🌐 Выбери DNS-серверы:"
echo "  1) Яндекс"
echo "  2) Google"
echo "  3) Cloudflare"
echo "  4) Ввести вручную"
read -p "👉 Введи номер (1-4): " dns_choice

case "$dns_choice" in
  1) DNS1="77.88.8.8"; DNS2="77.88.8.1" ;;
  2) DNS1="8.8.8.8"; DNS2="8.8.4.4" ;;
  3) DNS1="1.1.1.1"; DNS2="1.0.0.1" ;;
  4)
    read -p "📝 Введи первый DNS: " DNS1
    read -p "📝 Введи второй DNS: " DNS2
    ;;
  *) echo "⚠️ Неверный выбор. Используем Google."; DNS1="8.8.8.8"; DNS2="8.8.4.4" ;;
esac
echo "✅ Используем DNS: $DNS1 и $DNS2"

# Определим внешний интерфейс
EXT_IF=$(ip route | grep default | awk '{print $5}' | head -n1)
echo "🌐 Внешний интерфейс: $EXT_IF"

# Установка pptpd
apt update
apt install -y pptpd

# Конфигурация pptpd
cat > /etc/pptpd.conf <<EOF
localip 10.6.0.1
remoteip 10.6.0.100-200
EOF

# PPP настройки
cat > /etc/ppp/pptpd-options <<EOF
name pptpd
refuse-pap
refuse-chap
refuse-mschap
require-mschap-v2
require-mppe-128
ms-dns $DNS1
ms-dns $DNS2
proxyarp
lock
nobsdcomp
EOF

# Включение IP forwarding
echo "net.ipv4.ip_forward=1" >> /etc/sysctl.conf
sysctl -p

# NAT
iptables -t nat -A POSTROUTING -o "$EXT_IF" -j MASQUERADE
iptables-save > /etc/iptables.rules
cat > /etc/network/if-pre-up.d/iptables <<EOF
#!/bin/sh
iptables-restore < /etc/iptables.rules
EOF
chmod +x /etc/network/if-pre-up.d/iptables

# Настройка UFW
echo "🛡 Настройка UFW..."
if ! command -v ufw &> /dev/null; then
  echo "📥 Устанавливаем UFW..."
  apt install -y ufw
fi
ufw allow OpenSSH
ufw allow 1723/tcp
ufw allow proto gre
ufw status | grep -q inactive && {
  echo "🔔 Включаем UFW..."
  ufw --force enable
}
echo "✅ UFW настроен."

# Файлы пользователей
USERS_FILE="/etc/ppp/vpn_users.txt"
CHAP_FILE="/etc/ppp/chap-secrets"
touch "$USERS_FILE"
chmod 600 "$USERS_FILE"

# Скрипт обновления пользователей
cat > /usr/local/bin/update_vpn_users.sh <<'EOF'
#!/bin/bash
USERS_FILE="/etc/ppp/vpn_users.txt"
CHAP_FILE="/etc/ppp/chap-secrets"
> "$CHAP_FILE"
while IFS=' ' read -r user pass; do
  [ -z "$user" ] && continue
  echo "$user * $pass *" >> "$CHAP_FILE"
done < "$USERS_FILE"
systemctl restart pptpd
echo "✅ Пользователи обновлены и pptpd перезапущен."
EOF

chmod +x /usr/local/bin/update_vpn_users.sh

# Создаём первого пользователя
VPN_USER="vpn$(shuf -i 1000-9999 -n 1)"
VPN_PASS="$(tr -dc A-Za-z0-9 </dev/urandom | head -c 10)"
echo "$VPN_USER $VPN_PASS" >> "$USERS_FILE"
/usr/local/bin/update_vpn_users.sh

# Запускаем службы
systemctl restart pptpd
systemctl enable pptpd

echo ""
echo "✅ Установка завершена!"
echo "👤 Пользователь: $VPN_USER"
echo "🔐 Пароль:     $VPN_PASS"
echo "📄 Добавление: echo 'user pass' >> $USERS_FILE && /usr/local/bin/update_vpn_users.sh"
